/**
 *   File: timcard.h
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#ifndef CIEMATCMSTIMCARD_H
#define CIEMATCMSTIMCARD_H

#include "i_vmemaster.h"
#include "pca9564.h"
#include "ds2482.h"
#include "ds2438.h"
#include "i_vmedevice.h"
#include "logger.h"
#include "nstring.h"

#define TIMI2CCONTROLITEM  "TIMPCACONTROL"
#define TIMI2CDATAITEM     "TIMPCADATA"
#define TIMI2CSTATUSITEM   "TIMPCASTATUS"

namespace CiematCMS {

/**
	@author J.M. <josemanuel.cela@ciemat.es>
*/
class TIMCard : public I_VMEMaster {

  public:
    TIMCard(I_VMEDevice &deviceIn);
    ~TIMCard();

    /* Identificadores de los diferentes sensores. Repectivamente:
    sensor de 5V, de 3.3 V y de 1.8 V */
    enum SENSORS_ID {
      SENSIDVME5, SENSIDVME3_3, SENSIDTTC
    };

    void writeVME (string vmeAdd, unsigned char byte)
        throw (CMSException);
    void readVME (string vmeAdd, unsigned char* byte)
        throw (CMSException);

    void getTempVoltCurr(SENSORS_ID sensor,
                         float *temp, float *volt, float *current);

    unsigned long getSensorElapsedTime(SENSORS_ID sensor);

  private:
    /* Valor, a capn, de la direccin de esclavo del DS2482
    en el formato que su datasheet especifica */
    static unsigned int const ds2482_I2CSlaveADD=0x18;
    /* Valor (comn) de la resistencia que usan los sensores DS2438
    medida en ohms */
    static float const resistenciaSensor=5100;

    // Interfaz de conexin con el bus VME
    I_VMEDevice &device;
    // Diferentes integrados que componen la TIM.
    PCA9564 timPCA9564;
    DS2482  timDS2482;
    DS2438  timDS2438;
};

}

#endif
