/**
 *   File: vmedevice.h
 *
 *   Copyright (C) 2007 by J.M.
 *   josemanuel.cela@ciemat.es
 */
#ifndef CIEMATCMSVMEDEVICE_H
#define CIEMATCMSVMEDEVICE_H

#include "cmsexception.h"
#include "logger.h"
#include "nstring.h"
#include "i_configtree.h"
#include <libvme.h>

namespace CiematCMS {

/** Clase provisional que acta de interfaz con la librera LINCO,
 * sustituyendo a la que suministra la librera HAL.
 * 
 * El propsito de la misma es no tener que tocar el cdigo ya creado
 * para HAL. Debera bastar con sustituir las llamadas a la clase de HAL
 * por las llamadas a esta.
 *
 * 	@author J.M. <josemanuel.cela@ciemat.es>
 */
class VMEDevice{

  public:
    VMEDevice(I_ConfigTree &configTree, unsigned long VMEBaseAddress,
              int count, char space, int image, int bus);
    ~VMEDevice();

    void write(string offset, unsigned long valor)
        throw (CMSException);

    void read(string offset, unsigned long *valor)
        throw (CMSException);

    void setVMEBaseAddress(unsigned long address);
    unsigned long getVMEBaseAddress(void);

  private:
    /* Empotramos un objeto que proporcione la conexin via LINCO */
    //VMEBus vmebus;

    unsigned long VMEBaseAddress;

    /* Convierte el "string" que representa el OFFSET del registro
    a un valor numrico que incorpora la direccin base mas el
    propio offset.
    Este mtodo ser empleado por "write" y "read" */
    int convertAddress2LINCO (string VMEAddressOffset);

    VMEBus vmeController;
    I_ConfigTree &configTree;
};

}

#endif
