/**
 *   File: logger.cpp
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#include "logger.h"

namespace CiematCMS {

//**********************************************************
//*********** MIEMBROS ESTTICOS **************
//**********************************************************
/** Define el objeto DEFAULT_LOGGER nico y comn para todo el
 * cdigo. No es un mtodo sino un componente-miembro de la clase.
 *
 * Tal vez sea interesante introducir esta lnea en un "#ifdef" para
 * condicionar su compilacin a la definicin de la variable __DEBUG__
 * (ver fichero de logger.h), de manera que *NO* se cree este objeto
 * en caso que no se desee incluir cdigo de depuracin.
 * En la actualidad, puesto que es tipo "static" se crea este objeto de
 * forma global en cualquier caso.
 */
Logger Logger::defaultLogger(DEF_LOGGER_NAME);

//**********************************************************
//*********** CONSTRUCTORES Y DESTRUCTORES **************
//**********************************************************
Logger::Logger(string name)
  : logName(""), timeAuxiliar()
{
  logName= name;
  showLogName= true;
  showDateTime= true;

  string level="";
  char *levelEnv=NULL;

  /* Esta verificacin es necesaria puesto que, si no est
  definida la variable en el entorno, el sistema casca con un SIGSEGV.
  Se ve que la clase "string" estandar es un poco lela identificando
  anomalas en el "append" */
  levelEnv=getenv(ENV_LEVEL);
  if (levelEnv != NULL) level.append (levelEnv);

  if (level=="ALL")         currentLogLevel=LOG_LEVEL_ALL;
  else if (level=="TRACE")  currentLogLevel=LOG_LEVEL_TRACE;
  else if (level=="DEBUG")  currentLogLevel=LOG_LEVEL_DEBUG;
  else if (level=="INFO")   currentLogLevel=LOG_LEVEL_INFO;
  else if (level=="WARN")   currentLogLevel=LOG_LEVEL_WARN;
  else if (level=="ERROR")  currentLogLevel=LOG_LEVEL_ERROR;
  else if (level=="FATAL")  currentLogLevel=LOG_LEVEL_FATAL;
  else if (level=="OFF")    currentLogLevel=LOG_LEVEL_OFF;
  else currentLogLevel=LOG_LEVEL_WARN; // Opcion por defecto.
}

Logger::~Logger(){
}

//**********************************************************
//*********** MTODOS PBLICOS **************
//**********************************************************

void Logger::trace(string mensaje){
  if(isLevelEnable(LOG_LEVEL_TRACE))
     this->log(LOG_LEVEL_TRACE, mensaje);
}

void Logger::debug(string mensaje){
  if(isLevelEnable(LOG_LEVEL_DEBUG))
     this->log(LOG_LEVEL_DEBUG, mensaje);
}

void Logger::info(string mensaje){
  if(isLevelEnable(LOG_LEVEL_INFO))
     this->log(LOG_LEVEL_INFO, mensaje);
}
void Logger::warn(string mensaje){
  if(isLevelEnable(LOG_LEVEL_WARN))
     this->log(LOG_LEVEL_WARN, mensaje);
}

void Logger::error(string mensaje){
  if(isLevelEnable(LOG_LEVEL_ERROR))
     this->log(LOG_LEVEL_ERROR, mensaje);
}

void Logger::fatal(string mensaje){
  if(isLevelEnable(LOG_LEVEL_FATAL))
     this->log(LOG_LEVEL_FATAL, mensaje);
}

Logger &Logger::getdefaultLogger(void){
  return Logger::defaultLogger;
}

//**********************************************************
//*********** MTODOS GET Y SET **************
//**********************************************************
void Logger::setLevel(LOG_LEVELS level) {
  currentLogLevel=level;
}
Logger::LOG_LEVELS Logger::getLevel(void) {
  return currentLogLevel;
}

void Logger::showTime(void){showDateTime= true;}
void Logger::hideTime(void){showDateTime= false;}
void Logger::showName(void){showLogName= true;}
void Logger::hideName(void){showLogName= false;}

//**********************************************************
//*********** MTODOS PRIVADOS **************
//**********************************************************
/** Determina si un cierto nivel de log esta activo.
 *
 * Compara el nivel de log pasado como parametro con el que se
 * encuentra activo por defecto.
 * Esto es til para que cada mtodo-nivel que "loggea" consulte
 * primero si ha de incluir el mensaje pasante a la cadena de log.
 */
bool Logger::isLevelEnable(LOG_LEVELS level){
  return (level >= currentLogLevel);
}

/** Muestra la cadena de LOG en la salida standard de error
 */
void Logger::write(string cadena){
  cerr << cadena << endl;
}

/** Formatea la cadena de LOG
 *
 * Crea un string con el formato adecuado, incorporando el mensaje
 * que se pasa como parmetro.
 */
void Logger::log(LOG_LEVELS nivel, string mensaje) {

  // Una cadena temporal para guardar el log.
  string buffer;
  // Mostramos la fecha en caso que est activa.
  if(showDateTime) {
    string fecha;
    fecha = timeAuxiliar.getActualTime();
    buffer.append(fecha);
    buffer.append(" ");
  }
  // Aadimos un identificador del tipo de nivel de traza.
  switch(nivel) {
    case LOG_LEVEL_TRACE:
      buffer.append("[TRACE] ");
      break;
    case LOG_LEVEL_DEBUG:
      buffer.append("[DEBUG] ");
      break;
    case LOG_LEVEL_INFO:
      buffer.append("[INFO] ");
      break;
    case LOG_LEVEL_WARN:
      buffer.append("[WARN] ");
      break;
    case LOG_LEVEL_ERROR:
      buffer.append("[ERROR] ");
      break;
    case LOG_LEVEL_FATAL:
      buffer.append("[FATAL] ");
      break;
  }

  if(showLogName){
    buffer.append(logName);
    buffer.append(" - ");
  }

  // Append the message
  buffer.append(mensaje);

  write(buffer);
}

}
