/**
 *   File: ds2450.h
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#ifndef CIEMATCMSDS2450_H
#define CIEMATCMSDS2450_H

#include "dallas_1wireslave.h"
#include "logger.h"

namespace CiematCMS {

/** Implementacin de la lgica del sensor DS2450
 *
 * @author J.M. <josemanuel.cela@ciemat.es>
 */
class DS2450 : public Dallas_1WireSlave
{
  public:
    DS2450(I_OneWireMaster &master);
    ~DS2450();

    // Identificadores para los canales del integrado.
    enum INPUT_CHANNELS {
      AIN_A=0x01, AIN_B=0x02, AIN_C=0x04, AIN_D=0x08
    };
    /* Identificadores para los rangos mximos de tensin que permite
    el integrado.
    Indicamos explcitamente los valores del los enumerados para una
    mayor claridad, puesto que es el valor que ha de tener el bit-IR
    correspondiente en el registro de control */
    enum MAX_RANGE {
      MAX2_55V=0, MAX5_10V=1
    };

    void setInputChannel(enum INPUT_CHANNELS canal);
    /* Permite fijar el rango en un canal concreto */
    void setChannelMaxRange(enum INPUT_CHANNELS canal,
                            enum MAX_RANGE rango);

  private:
    /* Identifica el canal en curso sobre el que se van a efectuar
    las medidas */
    enum INPUT_CHANNELS currentChannel;

};

}

#endif
