/**
 *   File: ds2438.h
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#ifndef CIEMATCMSDS2438_H
#define CIEMATCMSDS2438_H

#include "dallas_1wireslave.h"
#include <unistd.h>

namespace CiematCMS {

/**
 * Implementacin de las primitivas tiles del sensor DS2438
 *
 * @author J.M. <josemanuel.cela@ciemat.es>
 */
class DS2438 : public Dallas_1WireSlave {

  public:
    DS2438(I_OneWireMaster &master);
    ~DS2438();
    void getTempVoltCurr (float *temp, float *volt, float *curr);

    void setSensorResistance(float dato);
    float getSensorResistance() const;

  private:
    /*  Duracin mxima estimada para cualquier conversin.
        Se usar como tiempo de espera en ciertos mtodos, para
        aguardar a que acabe la conversin de la temperatura o de la
        tesin.
        Est medido en microsegundos.
    */
    static unsigned int const WTIMECONVERT=50000;
    // Longitud de una pagina de memoria en bytes.
    static unsigned int const PAGELENGTH=9;

    enum CMD_CODES {
      CPY_SCRPAGE=0x48, CONVERT_T=0x44,
      CONVERT_V=0xB4,
      RCLL_MEMPAGE=0xB8, RD_SCRPAGE=0xBE,
      WR_SCRPAGE=0x4E
    };

    /* Valor de la resistencia conectada al sensor de corriente.
    Consultar datasheet del integrado */
    float sensorResistance;

    // Vector para el almacenamiento de una pagina de memoria
    // Mediante una union se le aade un molde para extraer comodamente
    // los bytes de datos que lo forman.
    union {
      unsigned char memPageData [PAGELENGTH];

      struct {
        unsigned char statConf, tempLSB, tempMSB,
        voltLSB, voltMSB,
        currLSB, currMSB,
        threshold;
      };
    } pagina0;

    void convert (float *temp, float *volt, float *curr);
    void readPage (unsigned pageNum);
};

}

#endif
