/**
 *   File: dallas_1wireslave.cpp
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#include "dallas_1wireslave.h"

namespace CiematCMS {

//**********************************************************
//*********** CONSTRUCTORES Y DESTRUCTORES **************
//**********************************************************
Dallas_1WireSlave::Dallas_1WireSlave(I_OneWireMaster &master)
  : I_OneWireSlave(master) {
  own1WROMAddress=0;
}

Dallas_1WireSlave::~Dallas_1WireSlave() {
}

//**********************************************************
//*********** MTODOS PBLICOS **************
//**********************************************************
/** Envia el comando SKIP-ROM por el bus 1-Wire, siguiendo las
 * especificaciones del protocolo.
 *
 * Es til para una comunicacin rpida en el caso en que *SOLO*
 * exista un dispositivo esclavo en el bus. En caso que haya ms de uno
 * se producir colisin y fallar todo procedimiento posterior.
 * \fn Dallas_1WireSlave::skipRom(void)
 */
void Dallas_1WireSlave::skipRom(void) {
    master1W.write1WByte(SKIP_ROM);
}

/** Enva por el bus 1-Wire, mediante el comando MATCH-ROM, la direccion
 * del esclavo que tendr que atender la siguiente peticin (orden)
 * que sea emitida.
 * \fn Dallas_1WireSlave::matchRom(void) throw (CMSException)
 */
void Dallas_1WireSlave::matchRom(unsigned long slaveAdd)
    throw (CMSException)
{
  // Enviamos el comando de MEMORIA
  master1W.write1WByte(MATCH_ROM);

  /* Recorremos bit a bit el "slaveAdd" y lo vamos enviando
  al bus. Para ello creamos una variable con el bit-0 a 1 y el resto
  a 0, y vamos rotando el bit a cada paso */
  unsigned long mascara=0x1;

  for (int i=0; i<= 63; i++) {
    if (slaveAdd & mascara) master1W.singleBit1WWrite(true);
    else master1W.singleBit1WWrite(false);

    mascara= mascara << 1;
  }
  /* Mandamamos una generosa excepcion si lo invocan hasta que lo hayamos
  probado suficientemente */
  throw (CMSException(
    "Dallas_1WireSlave::matchRom(void)",
    "!! CAGADA: Metodo sin probar. Comenta la excepcion y prubalo !!", 0)
    );
}

//**********************************************************
//*********** MTODOS GET Y SET **************
//**********************************************************
void Dallas_1WireSlave::setROMAddress(unsigned long add){
  own1WROMAddress= add;
}

unsigned long Dallas_1WireSlave::getROMAddress() const{
  return own1WROMAddress;
}

}
