/**
 *   File: roscard.h
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#ifndef CIEMATCMSROSCARD_H
#define CIEMATCMSROSCARD_H

#include "i_vmemaster.h"
#include "pca9564.h"
#include "ds2482.h"
#include "ds2438.h"
#include "i_vmedevice.h"
#include "nstring.h"

using namespace std;

/* Definiciones de los ITEMs que deben figurar en el fichero de
configuracin de HAL */
#define ROSI2CCONTROLITEM  "ROSPCACONTROL"
#define ROSI2CDATAITEM     "ROSPCADATA"
#define ROSI2CSTATUSITEM   "ROSPCASTATUS"
#define ROSGOLQPLLCTRL     "ROSGOLQPLL"

namespace CiematCMS {

/**
	@author J.M. <josemanuel.cela@ciemat.es>
*/
class ROSCard : public I_VMEMaster {

  public:
    ROSCard(I_VMEDevice &deviceIn);
    ~ROSCard();

    /* Identificadores de los diferentes sensores. Repectivamente:
    sensor de 5V, de 3.3 V y de 1.8 V */
    enum SENSORS_ID {
      SENSID5, SENSID3_3, SENSID1_8
    };

    void writeVME (string vmeAdd, unsigned char byte)
        throw (CMSException);
    void readVME (string vmeAdd, unsigned char* byte)
        throw (CMSException);

    /* Da las ordenes adecuadas a la FPGA para que se pueda
    acceder a los sensores en vez de al GOL. Tal vez convenga
    declararla como "private" o "protected" */
    void habilitarSensores (void) throw (CMSException);
    void getTempVoltCurr(SENSORS_ID sensor,
                         float *temp, float *volt, float *current);

    unsigned long getSensorElapsedTime(SENSORS_ID sensor);

  private:
    /* Valor, a capn, de la direccin de esclavo del DS2482
    en el formato que su datasheet especifica */
    static unsigned int const ds2482_I2CSlaveADD=0x18;
    /* Valor (comn) de la resistencia que usan los sensores DS2438
    medida en ohms */
    static float const resistenciaSensor=5100;

    // Interfaz de conexin con el bus
    I_VMEDevice &device;
    // Diferentes integrados que componen la ROS.
    PCA9564 rosPCA9564;
    DS2482  rosDS2482;
    DS2438  rosDS2438;
};

}

#endif
