/**
 *   File: timcard.cpp
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#include "timcard.h"

namespace CiematCMS {

//**********************************************************
//*********** CONSTRUCTORES Y DESTRUCTORES **************
//**********************************************************
TIMCard::TIMCard(I_VMEDevice &deviceIn)
  : I_VMEMaster(), device(deviceIn),
    timPCA9564( (*this) ),
    timDS2482(timPCA9564),
    timDS2438(timDS2482)
{
  DEFLOGINFO("Creamos un objeto TIMCard");

  // Configuramos los "ITEMs"
  timPCA9564.setControlRegItem(TIMI2CCONTROLITEM);
  timPCA9564.setDataRegItem(TIMI2CDATAITEM);
  timPCA9564.setStatusRegItem(TIMI2CSTATUSITEM);

  /* Si hubiera varios esclavos unidos al bus, esta llamada convendra
  hacerla antes de peticin de informacin al esclavo correpondiente.
  Puesto que todos los sensores de la ROS comparten la misma direccin de
  esclavo, por estar en distintos canales del DS2482, podemos efectuar
  la invocacin en el constructor y ahorramos llamadas */
  timDS2482.setI2CSlaveAddress(ds2482_I2CSlaveADD);
}

TIMCard::~TIMCard() {
  DEFLOGINFO("Destruimos un objeto TIMCard");
}

//**********************************************************
//*********** MTODOS PBLICOS **************
//**********************************************************
void TIMCard::writeVME (string vmeAdd, unsigned char byte)
    throw (CMSException)
{
  device.write(vmeAdd, (unsigned long)byte);

  DEFLOGTRACE(
    ((NString)("TIMCard.writeVME: byte escrito->")).\
    nAppend("%#X", (unsigned long)byte)
             );
}

void TIMCard::readVME (string vmeAdd, unsigned char* byte)
    throw (CMSException)
{
  // Variable til para la conversin de tipo.
  unsigned long valor=0;
  //uint32_t valor=0;

  device.read(vmeAdd, &valor);
  // Transcribimos el valor ledo al parmetro entrante.
  *byte=(unsigned char)valor;

  DEFLOGTRACE(
    ((NString)("TIMCard.readVME: byte leido->")).\
    nAppend("%#X", *byte));
}

/** Obtiene los parmetros del sensor que se indique.
 */
void TIMCard::getTempVoltCurr(SENSORS_ID sensor,
                              float *temp, float *volt, float *current)
{
  DEFLOGINFO("TIMCard: Entramos en getTempVoltCurr");

  /* Seleccionamos la entrada VAD o VDD (segn la tensin linea de
  tensin que se vaya a medir) del DS2438 y fijamos el canal del DS2482
  que queremos leer.

  Adems indicamos por separado el valor de la resistencia conectada al
  sensor.
  OJO! Esto se podra hacer en el constructor de RosCARD, pero obligara
  a que todos los posibles sensores de la tarjeta tuvieran conectada
  la misma resistencia. De momento esto es as, pero puede cambiar con
  la versin de la misma */
  timDS2438.setSensorResistance(resistenciaSensor);

  switch (sensor) {
    case SENSIDVME5:
      timDS2438.selectVDDInput();
      timDS2482.setChannel(DS2482::CH1);
      break;
    case SENSIDVME3_3:
      timDS2438.selectVADInput();
      timDS2482.setChannel(DS2482::CH0);
      break;
    case SENSIDTTC:
      timDS2438.selectVDDInput();
      timDS2482.setChannel(DS2482::CH2);
      break;

    default:
      timDS2438.selectVADInput();
      timDS2482.setChannel(DS2482::CH1);
  }
  timDS2438.getTempVoltCurr(temp, volt, current);

  DEFLOGINFO("TIMCard: Salimos de getTempVoltCurr");
}

/** Consulta al sensor correspondiente y devuelve el tiempo que
 * lleva encendido.
 *
 * (Ver comentarios del mtodo "getTempVoltCurr")
 */
unsigned long TIMCard::getSensorElapsedTime(SENSORS_ID sensor){

  DEFLOGINFO("TIMCard: Entramos en getSensorElapsedTime");

  unsigned long time= 0;

  switch (sensor) {
    case SENSIDVME5:
      timDS2482.setChannel(DS2482::CH1);
      break;
    case SENSIDVME3_3:
      timDS2482.setChannel(DS2482::CH0);
      break;
    case SENSIDTTC:
      timDS2482.setChannel(DS2482::CH2);
      break;

    default:
      timDS2482.setChannel(DS2482::CH1);
  }

  time=timDS2438.getElapsedTime();

  DEFLOGINFO("TIMCard: Salimos de getSensorElapsedTime");

  return time;
}

}
