/**
 *   File: $FILENAME$.cpp
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <iostream>
#include <cstdlib>
#include <fstream>
#include <stdlib.h>
#include <unistd.h>

#include "cmsexception.h"
#include "roscard.h"
#include "timcard.h"
#include "logger.h"
#include "nstring.h"
#include "vmedevice.h"
#include "vmedevicedummy.h"
#include "configfileparser.h"
#include "i_configtree.h"

#include <libvme.h>

using namespace std;
using namespace CiematCMS;

#define _DEFNUMCICLO 100
#define _DEFPERIODO 60
#define _DEFSALIDA "salida.txt"
#define _DEFCONFIGFILE "config.txt"

/* Esto es una chapuza infecta. Habra que crear una clase
y encapsular las cosas en condiciones, pero no quiero perder
ms tiempo. As que basura-codigo al canto! */
// Numero maximo de ROS
#define MAXROS 6 

// Proto-pitos.
void configura (ConfigFileParser &parser, ROSCard **ROS, TIMCard **TIM,
                string &outFile, unsigned long int &numCiclos,
                int &periodo);

NString leeRos(ROSCard &ROS);
void leeSensorROS(ROSCard &ROS, ROSCard::SENSORS_ID sensorId,
               float *temp, float *volt, float *current,
               unsigned long int *tiempo);

NString leeTim(TIMCard &TIM);
void leeSensorTIM(TIMCard &TIM, TIMCard::SENSORS_ID sensorId,
               float *temp, float *volt, float *current,
               unsigned long int *tiempo);

bool pruebaFichero(string fichNom);

int main(int argc, char *argv[]) {

  // Array para las 6 posibles ROS de un crate.
  ROSCard *ROS[MAXROS];
  // Puntero a la TIM.
  TIMCard *TIM = NULL;

  try {
    // En esta zona hay que hacer todos los preparativos para el
    // acceso a la librera LINCO
    string fichconf;
    if ( ! pruebaFichero(_DEFCONFIGFILE) ) {
      cout << "==== Configuracion de acceso a sensores ====" << endl;
      cout << "-> Introduzca el nombre del fichero de configuracin: ";
      cin >> fichconf;
    }
    else { fichconf = _DEFCONFIGFILE; }

    //********* Recuperamos la informacin de configuracin.
    unsigned long int numCiclos=0;
    int periodo=0;
    string outFile;

    /* Creamos en este mbito el parser, puesto que ha de estar "vivo"
    durante toda la ejecucin del cdigo, para poder ser usado por las
    clases que necesitan datos de configuracin */

    ConfigFileParser parser(fichconf);
    parser.parseFile();

    configura(parser, ROS, &TIM, outFile, numCiclos, periodo);

    ofstream salida(outFile.c_str());

    //********* Comenzamos la toma de datos.
    TimeGeneral hora;
    salida << "==== CRATE DTE. TOMA DE DATOS DE SENSORES ====" << endl;
    salida << "==== " << endl;

    cout << "-- Procedemos a la toma de datos." << endl;
    int rosnum = 0;
    unsigned long int ciclo = 0;
    do {
      cout << "-> Ciclo: " << ciclo++ << endl;
      salida << "==== Hora: " << hora.getActualTime() << endl;
      // Procesamos las 6 ROS.
      for (rosnum=0; rosnum < MAXROS; rosnum ++){
        if ( ROS[rosnum] == NULL ) continue;

        salida << "== ROS: " << rosnum << endl;
        salida << leeRos( *ROS[rosnum] ) << endl << "==" << endl << endl;
        salida.flush();
      }

      // Procesamos la TIM si est definida.
      if ( TIM != NULL ) {
        salida << "== TIM. " << endl;
        salida << leeTim( *TIM ) << endl << "==" << endl << endl;
        salida.flush();
      }

      usleep (periodo * 1000000);
    } while (ciclo < numCiclos);

  }
  catch (CMSException e) {
    cout << "Guay!. Hemos petao con una seora excepcion." << endl <<
        "La traza de error es:" << endl;
    cout << e.getMensaje() << endl;

  }
  catch (VME_BusError e) {
    cout << "Excepcion VME de la librera."   << endl;
    cout << "Address: " << e.address << ", count: " << e.count <<
        ", type: " << e.access_type << endl;
  }
  catch (std::exception e) {
    cout << "Excepcion estandard." << endl;
    cout << "Error: " << e.what()  << endl;
  }


  /* Habra que hacer "delete" de todos los objetos creados durante
  la configuracin de las ROSCard, pero se lo dejo al S.O. */

  return 0;
}

/**
 * Damos la orden de lectura a todos los sensores y devolvemos una cadena
 * formateada
 */
NString leeRos(ROSCard &ROS)
{
  float temp=0, volt=0, current=0, currCompens = 0;
  unsigned long int time= 0;

  NString lectura;

  leeSensorROS(ROS, ROSCard::SENSID5, &temp, &volt, &current, &time);
  currCompens = current*101666;
  lectura.sAppend("-- Sensor 5.0\n").\
      sAppend("   Temp(C): ").nAppend("%f", temp).\
      sAppend("; Volt(V): ").nAppend("%f", volt).\
      sAppend("; Curr(A): ").nAppend("%f", current).\
      sAppend("; Curr(A->Comp.): ").nAppend("%f", currCompens).\
      sAppend("; Time(seg): ").nAppend("%i", time).\
      sAppend("\n");

  leeSensorROS(ROS, ROSCard::SENSID3_3, &temp, &volt, &current, &time);
  currCompens = current*101666;
  lectura.sAppend("-- Sensor 3.3\n").\
      sAppend("   Temp(C): ").nAppend("%f", temp).\
      sAppend("; Volt(V): ").nAppend("%f", volt).\
      sAppend("; Curr(A): ").nAppend("%f", current).\
      sAppend("; Curr(A->Comp.): ").nAppend("%f", currCompens).\
      sAppend("; Time(seg): ").nAppend("%i", time).\
      sAppend("\n");

  leeSensorROS(ROS, ROSCard::SENSID1_8, &temp, &volt, &current, &time);
  currCompens = current*101666;
  lectura.sAppend("-- Sensor 1.8\n").\
      sAppend("   Temp(C): ").nAppend("%f", temp).\
      sAppend("; Volt(V): ").nAppend("%f", volt).\
      sAppend("; Curr(A): ").nAppend("%f", current).\
      sAppend("; Curr(A->Comp.): ").nAppend("%f", currCompens).\
      sAppend("; Time(seg): ").nAppend("%i", time).\
      sAppend("\n");

  return lectura;
}

/**
 * Recuperamos los datos de los sensores de una ROS
 */
void leeSensorROS(ROSCard &ROS, ROSCard::SENSORS_ID sensorId,
               float *temp, float *volt, float *current,
               unsigned long int *tiempo){
  ROS.getTempVoltCurr(sensorId, temp, volt, current);
  *tiempo = ROS.getSensorElapsedTime(sensorId);
}

/**
 * Damos la orden de lectura a todos los sensores y devolvemos una cadena
 * formateada
 */
NString leeTim(TIMCard &TIM)
{
  float temp=0, volt=0, current=0, currCompens = 0;
  unsigned long int time= 0;

  NString lectura;

  leeSensorTIM(TIM, TIMCard::SENSIDVME5, &temp, &volt, &current, &time);
  currCompens = current*101666;
  lectura.sAppend("-- Sensor 5.0\n").\
      sAppend("   Temp(C): ").nAppend("%f", temp).\
      sAppend("; Volt(V): ").nAppend("%f", volt).\
      sAppend("; Curr(A): ").nAppend("%f", current).\
      sAppend("; Curr(A->Comp.): ").nAppend("%f", currCompens).\
      sAppend("; Time(seg): ").nAppend("%i", time).\
      sAppend("\n");

  leeSensorTIM(TIM, TIMCard::SENSIDVME3_3, &temp, &volt, &current, &time);
  currCompens = current*101666;
  lectura.sAppend("-- Sensor 3.3\n").\
      sAppend("   Temp(C): ").nAppend("%f", temp).\
      sAppend("; Volt(V): ").nAppend("%f", volt).\
      sAppend("; Curr(A): ").nAppend("%f", current).\
      sAppend("; Curr(A->Comp.): ").nAppend("%f", currCompens).\
      sAppend("; Time(seg): ").nAppend("%i", time).\
      sAppend("\n");

  leeSensorTIM(TIM, TIMCard::SENSIDTTC, &temp, &volt, &current, &time);
  currCompens = current*101666;
  lectura.sAppend("-- Sensor 1.8\n").\
      sAppend("   Temp(C): ").nAppend("%f", temp).\
      sAppend("; Volt(V): ").nAppend("%f", volt).\
      sAppend("; Curr(A): ").nAppend("%f", current).\
      sAppend("; Curr(A->Comp.): ").nAppend("%f", currCompens).\
      sAppend("; Time(seg): ").nAppend("%i", time).\
      sAppend("\n");

  return lectura;
}

/**
 * Recuperamos los datos de los sensores de la TIM
 */
void leeSensorTIM(TIMCard &TIM, TIMCard::SENSORS_ID sensorId,
               float *temp, float *volt, float *current,
               unsigned long int *tiempo){
  TIM.getTempVoltCurr(sensorId, temp, volt, current);
  *tiempo = TIM.getSensorElapsedTime(sensorId);
}

/**
 * Lee los parametros del fichero de configuracion.
 */
void configura (ConfigFileParser &parser, ROSCard **ROS, TIMCard **TIM,
                string &outFile, unsigned long int &numCiclos,
                int &periodo){

  cout << "-> Determinando el intervalo entre lecturas." << endl;
  try {
    periodo = NString(parser.getValue("PERIODO")).convert2Int();
    cout << "Fijando periodo a: " << periodo << endl;
  }
  catch (CMSException e) {
    cout << "Valor de periodo erroneo o no especificado." << endl\
         << "Fijando periodo por defecto a " << _DEFPERIODO << " seg."\
         << endl;
    periodo = _DEFPERIODO;
  }

  cout << "-> Determinando el nombre del fichero de salida." << endl;
  try {
    outFile = parser.getValue("OUT");
    cout << "Fijando fichero de salida a: " << outFile << endl;
  }
  catch (CMSException e){
    cout << "Nombre de fichero de salida no especificado." << endl \
         << "Fijando fichero de salida a: " << _DEFSALIDA << endl;
    outFile = _DEFSALIDA;
  }

  cout << "-> Determinando el numero de ciclos de lectura." << endl;
  try {
    numCiclos = NString(parser.getValue("NUMCICLOS")).convert2Int();
    cout << "Fijando numero de ciclos a: " << numCiclos << endl;
  }
  catch (CMSException e){
    cout << "Numero de ciclos no especificado." << endl \
         << "Fijando numero de ciclos a: " << _DEFNUMCICLO << endl;
    numCiclos = _DEFNUMCICLO;
  }

  NString ROSID, TIMID;
  int addTemp, addRelTemp, crateNum, imageNum, spaceSize;

  try {
    // Leemos el identificador del crate.
    crateNum = NString(parser.getValue("CRATENUMBER")).convert2Int();
  }
  catch (CMSException e){
    cout << "No se localiza la variable: CRATENUMBER" << endl;
    throw e;
  }
  
  try {
    //... el tamao del espacio de memoria del crate
    spaceSize = NString(parser.getValue("SPACESIZE")).convert2Int();
  }
  catch (CMSException e){
    cout << "No se localiza la variable: SPACESIZE" << endl;
    throw e;
  }

  try {
    //... la direccin base del cacharro
    addTemp = NString(parser.getValue("CRATEBASEADDRESS")).convert2Int();
  }
  catch (CMSException e){
    cout << "No se localiza la variable: CRATEBASEADDRESS" <<
        endl;
    throw e;
  }
  try {
    //... y la valor del nmero de image.
    imageNum = NString(parser.getValue("IMAGENUM")).convert2Int();
  }
  catch (CMSException e){
    cout << "No se localiza la variable: IMAGENUM" << endl;
    throw e;
  }


  /* En esta nueva versin, creamos un slo VMEBus para todas
  las tarjetas */
  VMEBus *busControl = new VMEBus(addTemp,
      spaceSize, VME_ADDR16|VME_SIZE_D16, imageNum, crateNum);

  // Si una ROS no esta definida, metemos NULL en el puntero.
  for (int rosnum=0; rosnum < MAXROS; rosnum++) {
    try {
      ROSID=NString("ROS_");
      ROSID.nAppend("%i", rosnum);

      addRelTemp = NString(parser.getValue(ROSID)).convert2Int();

      cout << "-> Configurando ROS: " << rosnum <<\
          " con direccin base: " << addTemp <<\
          ", desplazamiento relativo: " << addRelTemp<<\
          ", espacio de memoria global: " << spaceSize <<\
          ", identificador de canal/crate: " << crateNum <<\
          ", y numero de imagen: " << imageNum << endl;

      //VMEDeviceDummy *device = new VMEDeviceDummy(parser,
      //    addTemp, addRelTemp);
      VMEDevice *device = new VMEDevice(parser, *busControl, addTemp,
                                        addRelTemp);

      ROS[rosnum] = new ROSCard( (*device) );
    }
    catch (CMSException e){
      // ROS no definida.
      ROS[rosnum] = NULL;
    }
  }

  // Configuramos la TIM, si est definida
  try {
    TIMID=NString("TIM");
    addRelTemp= NString(parser.getValue(TIMID)).convert2Int();

    cout << "-> Configurando TIM, con direccin base: "
        << addTemp <<\
        ", desplazamiento relativo: " << addRelTemp<<\
        ", espacio de memoria global: " << spaceSize <<\
        ", identificador de canal/crate: " << crateNum <<\
        ", y numero de imagen: " << imageNum << endl;

    //VMEDeviceDummy *device = new VMEDeviceDummy(parser,
    //    addTemp, addRelTemp);
    VMEDevice *device = new VMEDevice(parser, *busControl, addTemp,
                                      addRelTemp);

    *TIM = new TIMCard( (*device) );
  }
  catch (CMSException e){
    // TIM no definida.
    TIM = NULL;
  }
}

/* Devuelve TRUE si el fichero existe y es accesible */
bool pruebaFichero(string fichNom){

  ifstream strTemp;

  strTemp.open(fichNom.c_str());
  if ( !strTemp.is_open() ) return false;
  else {
    strTemp.close();
    return true;
  }
}
